<?php

namespace App\Service;

use App\Exception\Exception;
use App\Util\Command;
use App\Util\Printing;

class PrintService{

    private $printFactory;
    protected $printer;
    private $util;

    public function __construct(
        \App\Factory\PrintFactory $printFactory,
        \App\Util\Printer $printer,
        \App\Util\Util $util
    ) {
        $this->printFactory          = $printFactory;
        $this->comandos              = array();
        $this->util                  = $util;
        $this->utilizaImpressaoPonte = true;
        $this->printer               = $printer;        
    }

    const IMP_FONTE_EXPADIDA = ['13', '18', '21', '28', '34'];

    public function imprimePedido($cdFilial, $cdLoja, $produtos, $cdVendedor, $dsComanda, $nrMesa, $cdSenhaPed,
                                  $tipoVenda, $toGoString, $multiplasComandas, $ultimaComanda, $NRSEQVENDA = null,
                                  $cdCaixa, $IDIMPPEDPRCIGE, $NMCONSUMIDOR = null){

        $dadosLoja = $this->printFactory->getBranchDetails($cdFilial, $cdLoja);
        $IDLUGARMESA = $dadosLoja['IDLUGARMESA'];
        $IDPOSOBSPED = $dadosLoja['IDPOSOBSPED'];
        $IDUTLCORTEPED = $dadosLoja['IDUTLCORTEPED'];
        // TipoVenda B (balcão) do TAA não tem mesa.
        $NMSALA = null;
        if (!empty($nrMesa)) {
            $dadosMesa = $this->printFactory->getTableDetails($cdFilial, $cdLoja, $nrMesa);
            $nmSala = $dadosMesa['NMSALA'];
        }

        $dadosVendedor = $this->printFactory->getSellerDetails($cdVendedor);
        $dadosFilial = $this->printFactory->getGeneralParams($cdVendedor);
        if (!empty($dadosFilial) && $dadosFilial['IDIMPCOMPACTA'] === 'S') $compactPrint = true;
        else $compactPrint = false;

        if (!empty($dadosVendedor['NMFANVEN'])) $NMFANVEN = $dadosVendedor['NMFANVEN'];
        else $NMFANVEN = "";
        
        // Verifica parametro IDIMPRODUVEZ dos produtos para replicá-los para impressão.
        $produtos = $this->verificaImpUmaVez($produtos);
        // Reorganiza o array para que os produtos que serão impressos separados (IDIMPRODUVEZ) fiquem no final.
        $produtos = $this->alteraOrdemImpressao($produtos);

        $produtosIndexadosPorImpressora = $this->indexaProdutosPorImpressora($cdFilial, $cdLoja, $produtos);
        $impressoraProducao = $produtosIndexadosPorImpressora['impressoraProducao'];
        $impressoraProducao2 = $produtosIndexadosPorImpressora['impressoraProducao2'];
        $impressoraPuxa = $produtosIndexadosPorImpressora['impressoraPuxa'];

        $result = array(
            'error' => false
        );

        $IDUTILFONTEXP = $this->printFactory->getUseExpandedFont($cdFilial, $cdCaixa);
		$IDUTILFONTEXP = isset($IDUTILFONTEXP['IDUTILFONTEXP']) ? $IDUTILFONTEXP['IDUTILFONTEXP'] : 'N';

        foreach ($impressoraProducao as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = $this->montaPedido($impressoraAtual, $dsComanda, $nrMesa, $nmSala, $cdVendedor, $NMCONSUMIDOR,
            $NMFANVEN, $cdSenhaPed, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda, $NRSEQVENDA, $cdCaixa, $cdFilial, $produtos, $compactPrint);
            if ($respostaPedido['error']){
                $result = $respostaPedido;
            }
        }
        foreach ($impressoraProducao2 as $impressoraAtual){
            $cutByPosition = $IDUTLCORTEPED === 'S';
            $respostaPedido = $this->montaPedido($impressoraAtual, $dsComanda, $nrMesa, $nmSala, $cdVendedor, $NMCONSUMIDOR,
             $NMFANVEN, $cdSenhaPed, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda, $NRSEQVENDA, $cdCaixa, $cdFilial, $produtos, $compactPrint);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }
        foreach ($impressoraPuxa as $impressoraAtual){
            $respostaPedido = $this->montaPedido($impressoraAtual, $dsComanda, $nrMesa, $nmSala, $cdVendedor, $NMCONSUMIDOR,
             $NMFANVEN, $cdSenhaPed, $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, false, $multiplasComandas, $ultimaComanda, $NRSEQVENDA, $cdCaixa, $cdFilial, $produtos, $compactPrint);
            if ($respostaPedido['error']) $result = $respostaPedido;
        }

        // DEBUG - Descomente para visualizar a impressão no browser.
        // foreach($this->comandos as $printSheet){
        //     $text = "\n";
        //     foreach($printSheet['comandos']->getCommands() as $command){
        //         if ($command['name'] == "text"){
        //             $text .= $command['parameters']['text'];
        //         }
        //     }
        // };

        if($IDIMPPEDPRCIGE == 'S'){
            foreach($this->comandos as $printSheet){
                $text = "\n";
                foreach($printSheet['comandos']->getCommands() as $command){
                    if ($command['name'] == "text"){
                        $text .= $command['parameters']['text'];
                    }
                }
            };
            array_push($result, array('text' => $text));
        }

        if($ultimaComanda && $IDIMPPEDPRCIGE != 'S'){
            $dadosImpressora = $this->getDadosImpressora($cdFilial, $cdCaixa);
            $IDUTILIMPFRONT = $dadosImpressora['dadosImpressora']['IDUTILIMPFRONT'];
            $issaas = false;
            $result['paramsImpressora'] = [];

            foreach ($this->comandos as $comando){

                if ($IDUTILIMPFRONT =='S'){
                    array_push($result['paramsImpressora'],
                            array(
                                    'saas'           => false,
                                    'IDUTILIMPFRONT' => $IDUTILIMPFRONT=='S',
                                    'impressora'     => $comando['impressora'],
                                    'comandos'       => $comando['comandos']->getCommands(),
                                    'error'          => false
                            )
                            );
                    $respostaPonte['error'] = false;
                } else{
                    $printerParams = $this->buscaParametrosImpressora($comando['impressora']['IDMODEIMPRES']);
                    if (!$printerParams['impressaoFront']) {

                        if($IDUTILFONTEXP == 'S' && (in_array($comando['impressora']['IDMODEIMPRES'], self::IMP_FONTE_EXPADIDA))) {
                            $ultimaposicao=false;
                            $contador=false;
                            $boExpandiu=false;
                            foreach($comando['comandos']->getCommands() as &$command){
                                if($ultimaposicao===false) { 
                                    $contador = 0; 
                                    $ultimaposicao=0; 
                                }

                                if ($command['name'] == "text") {
                                    //$command['parameters']['text'] = $this->util->retornaTextoComCodigosEscPos($command['parameters']['text']);
                                    $command['parameters']['text'] = chr(27).chr(33).chr(16).$command['parameters']['text'];
                                    $boExpandiu = true;
                                    $ultimaposicao=$contador;
                                }

                                $contador = $contador+1;
                            }
                            if($boExpandiu) {
                                if($comando['comandos']->getCommands()[$ultimaposicao]['name']=='text') {
                                    $comando['comandos']->getCommands()[$ultimaposicao]['parameters']['text']= $comando['comandos']->getCommands()[$ultimaposicao]['parameters']['text'].chr(27).chr(64);
                                }
                            }
                        }
                            
                        $resultImp = $this->impressaoPedidos($comando['impressora'], $comando['comandos'], $dsComanda);
                        if(isset($resultImp['error']) && $resultImp['error']) {
                            $result = $resultImp;
                        }
                    }
                }
            }
            $this->comandos = array();
        }
        return $result;
    }

    private function montaPedido($impressoraAtual, $DSCOMANDA, $nrMesa, $nmSala, $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $CDSENHAPED,
                                 $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $cutByPosition, $multiplasComandas, $ultimaComanda,
                                 $NRSEQVENDA, $CDCAIXA, $CDFILIAL, $produtos, $compactPrint){

        $posicoes = array_unique(array_column($impressoraAtual['produtos'], 'NRLUGARMESA'));

        if(!isset($this->comandos[$impressoraAtual['NRSEQIMPRLOJA']])){
            $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'] = new Command();
            $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['impressora'] = $impressoraAtual;
        }

        if($cutByPosition){
            foreach ($posicoes as $posicao) {
                $retornoImprime =  $this->_imprimePedido($impressoraAtual, $DSCOMANDA, $nrMesa, $nmSala, $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $CDSENHAPED,
                    $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, array($posicao), $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'], $multiplasComandas, $NRSEQVENDA, $CDCAIXA, $CDFILIAL, $produtos, $compactPrint);
                if($retornoImprime['error']){
                    return $retornoImprime;
                }
            }
        } else {
            $retornoImprime = $this->_imprimePedido($impressoraAtual, $DSCOMANDA, $nrMesa, $nmSala, $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $CDSENHAPED,
                $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $posicoes, $this->comandos[$impressoraAtual['NRSEQIMPRLOJA']]['comandos'], $multiplasComandas, $NRSEQVENDA, $CDCAIXA, $CDFILIAL, $produtos, $compactPrint);
        }
        if($retornoImprime['error']){
            return $retornoImprime;
        }else{
            if($multiplasComandas){
                if($ultimaComanda){
                    foreach ($this->comandos as $comando) {
                        $comando['comandos']->cutPaper();
                    }
                }
                return array('error' => false);
            }else{
                return array('error' => false);
            }
        }
    }

    private function _imprimePedido($impressoraAtual, $DSCOMANDA, $nrMesa, $nmSala, $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $CDSENHAPED,
                                    $IDLUGARMESA, $IDPOSOBSPED, $tipoVenda, $toGoString, $posicoes, $comandos, $multiplasComandas, $NRSEQVENDA,
                                    $CDCAIXA, $CDFILIAL, $dadosPedido, $compactPrint){
        global $CLIENT_NAME;
        $this->logPath = $CLIENT_NAME['logPath'] . "/imp";

        $produtos = $impressoraAtual['produtos'];
        $tipoImpressao = $impressoraAtual['tipoImpressao'];

        $printerParams = $this->buscaParametrosImpressora($impressoraAtual['IDMODEIMPRES']);
        $printerParams['letterType'] = $printerParams['tipoLetra'];
        #TODO fazer validacao respeitando parametro IDUTILFONTEXP = 'S';

        $IDUTILFONTEXP = $this->printFactory->getUseExpandedFont($CDFILIAL, $CDCAIXA);
		$IDUTILFONTEXP = isset($IDUTILFONTEXP['IDUTILFONTEXP']) ? $IDUTILFONTEXP['IDUTILFONTEXP'] : 'N';

        //if($impressoraAtual['IDMODEIMPRES'] == "13") {
        //    $printerParams['largura'] = $printerParams['largura'];
        //    $printerParams['expanded'] = 1;
        //}
        
        //da pra otimizar
        $arrNmPosicao = $this->buscaNomePorPosicao($produtos, $posicoes);
        $textHeader = '';
        $textHeader = $this->printHeader($textHeader, $printerParams, $tipoVenda, $DSCOMANDA, $nrMesa, $nmSala,
                                         $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $tipoImpressao, $toGoString, $comandos, $compactPrint);

        //command header
        $printerParams['bold'] = true;
        $impSimulada = "{$textHeader}\n";
        $comandos->text($textHeader, $printerParams);

        // Define footer.
        $textFooter = $printerParams['comandoEnter'];
        if ($compactPrint){
            $textFooter .= $this->printFooter($textFooter, $printerParams, $CDSENHAPED, $compactPrint);
            $impSimulada .= "{$textFooter}\n";
        }

        foreach ($posicoes as $posicao) {
            //imprime a posição no pedido
            $textPosicao = '';
            if (($IDLUGARMESA === 'S') && ($tipoVenda === 'M')) {
                $this->checaEnter($textPosicao, $printerParams);
                $strPosicao = 'POSICAO ' . $posicao;

                if (isset($arrNmPosicao[$posicao])){
                    $strPosicao .= ' - ' . $arrNmPosicao[$posicao]['DSCONSUMIDOR'];
                    if (strlen($strPosicao) > $printerParams['largura']) {
                        $strPosicao = $this->quebraLinha($strPosicao, $printerParams, true);
                    }
                }
                $textPosicao .= $strPosicao . $printerParams['comandoEnter'];
            }
            //command posição
            $printerParams['bold'] = false;
            if($textPosicao != '') {
                $comandos->text($textPosicao, $printerParams);
            }
            $impSimulada .= "{$textPosicao}\n";
            $textProdutos = '';
            $ordemImp = 0;
            foreach ($produtos as $key => $produtoAtual) {
                //imprime produto
                if ($posicao == $produtoAtual['NRLUGARMESA'] && (($tipoVenda !== 'B' && (!empty($produtoAtual['CDPRODPROMOCAO']) || (empty($produtoAtual['CDPRODPROMOCAO'])))))){
                    if (!empty($produtoAtual['ORDEMIMP']) && ($produtoAtual['ORDEMIMP'] - $ordemImp >= 1) && ($ordemImp != 0)) {
                        // Quando acabar os produtos da promoção, é impressa uma linha.
                        $printerParams['bold'] = false;
                        $this->imprimeLinhaCommand($printerParams, $comandos);

                        $ordemImp = 0;
                        $impSimulada .= "{$textProdutos}\n";
                        $textProdutos = '';
                    }

                    // Caso o pedido seja impresso separado, imprime o rodapé e cabeçalho novamente.
                    if ($produtoAtual['IDIMPRODUVEZ'] === 'S'){
                        $comandos->text($textFooter, $printerParams);
                        $comandos->text($textHeader, $printerParams);
                    }

                    // Cabeçalho da promoção inteligente.
                    if ($produtoAtual['IDIMPPROMOCAO'] === 'S') {
                        $printerParams['bold'] = false;
                        $this->imprimeLinhaCommand($printerParams, $comandos);

                        $textProdutos .= $this->centraliza($printerParams, $produtoAtual['NMPRODPROMOCAO']) . $printerParams['comandoEnter'];
                        if (($produtoAtual['TXPRODCOMVEN'] !== " ") && ($produtoAtual['TXPRODCOMVEN'] != '')) {
                            // Imprime observação do produto pai.
                            $obsArray = explode(";", $produtoAtual['TXPRODCOMVEN']);
                            foreach ($obsArray as $obs){
                                if (!empty($obs)){
                                    $textProdutos .= '  * ' . trim($obs) . $printerParams['comandoEnter'];
                                }
                            }
                            $textProdutos .= $this->centraliza($printerParams, '* ' . $produtoAtual['TXPRODCOMVEN'] . $printerParams['comandoEnter']);
                            $produtoAtual['TXPRODCOMVEN'] = null;
                        }

                        $printerParams['bold'] = true;
                        $comandos->text($textProdutos, $printerParams);
                        $impSimulada .= "{$textProdutos}\n";
                        $textProdutos = '';

                        $printerParams['bold'] = false;
                        $textProdutos .= $this->imprimeLinhaCommand($printerParams, $comandos);

                        // Variável usada para imprimir a última linha da promoção inteligente.
                        $ordemImp = bcdiv(str_replace(',','.',strval($produtoAtual['ORDEMIMP'])), '1', '0');
                    }
                    
                    // Verifica se é para imprimir observações antes do produto.
                    if ($IDPOSOBSPED === 'A'){
                        if (($produtoAtual['TXPRODCOMVEN'] !== " ") && ($produtoAtual['TXPRODCOMVEN'] != '')){
                            $textProdutos .= '  *   ' . $produtoAtual['TXPRODCOMVEN'] . $printerParams['comandoEnter'];
                        }

                        if ($produtoAtual['ATRASOPROD']){
                            $textProdutos .= '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $printerParams['comandoEnter'];
                        }
                    }

                    // Imprime a linha do produto, exceto se for cabeçalho da promoção.
                    if ($produtoAtual['IDIMPPROMOCAO'] !== 'S') {
                        $textProdutos .= ' ---> ' . $produtoAtual['QTPRODCOMVEN'] . ' ' . self::removeAcentos($produtoAtual['NMPRODUTO']) . $printerParams['comandoEnter'];
                    }

                    // Imprime observação depois do produto.
                    if ($IDPOSOBSPED === 'D'){
                        if (($produtoAtual['TXPRODCOMVEN'] !== " ") && ($produtoAtual['TXPRODCOMVEN'] != '')){
                            $obsArray = explode(";", $produtoAtual['TXPRODCOMVEN']);
                            foreach ($obsArray as $obs){
                                if (!empty($obs)){
                                    $textProdutos .= '  * ' . trim($obs) . $printerParams['comandoEnter'];
                                }
                            }
                        }

                        if ($produtoAtual['ATRASOPROD']){
                            $textProdutos .= '   *  Segura de ' . $produtoAtual['ATRASOPROD'] . ' minutos' . $printerParams['comandoEnter'];
                        }
                    }

                    // Imprime indicação de que o produto é para viagem.
                    if (!empty($produtoAtual['IDORIGEMVENDA'])) {
                        $textProdutos .= '   *  Para viagem ' . $printerParams['comandoEnter'];
                    }

                    //tira do array pra nao precisar checar na proxima observacao
                    unset($produtos[$key]);
                    //command produto
                    if ($textProdutos != ''){
                        $printerParams['bold'] = true;
                        $printerParams['letterType'] = '2';
                        $impSimulada .= "{$textProdutos}\n";
                        $textProdutos .= $printerParams['comandoEnter'];
                        $comandos->text($textProdutos, $printerParams);
                        $textProdutos = '';
                    }
                    $printerParams['letterType'] = $printerParams['tipoLetra'];
                }
            }
            //sem tracinho :<
            //if($posicao == posicoes[count(posicoes)-1])
            //$this->impressaoUtil->imprimeLinhaCommand($printerParams, $comandos);
        }

        if (!$compactPrint){
            $textFooter = '';
            $textFooter = $this->printFooter($textFooter, $printerParams, $CDSENHAPED, $compactPrint);
            $impSimulada .= "{$textFooter}\n";
            $printerParams['bold'] = true;
            $comandos->text($textFooter, $printerParams);

            // GERAÇÃO DE CÓDIGO DE BARRAS COMENTADO (verificar utilidade antes de voltar)
            // $barCodeOptions = array(
            //     'height'   => 50,
            //     'width'    => 1,
            //     'position' => 0,
            //     'font'     => 0,
            //     'margin'   => 25
            // );

            // if (!empty($NRSEQVENDA))
            //     $comandos->barCode($CDCAIXA.$CDFILIAL.$NRSEQVENDA, $barCodeOptions);
            // else
            //     $comandos->barCode($CDFILIAL.$dadosPedido[0]["NRVENDAREST"].$dadosPedido[0]["NRCOMANDA"], $barCodeOptions);
        }

        $text = $printerParams['comandoEnter'];
        $text .= $printerParams['comandoEnter'];
        //$text .= $printerParams['comandoEnter'];
        //$text .= $printerParams['comandoEnter'];
        $espelho = $comandos->text($text, $printerParams);
        
        $folder = $this->logPath . "/pedidos/" . date("m") . "/" . date("d"). "/";

        if (!file_exists($folder)) {
            mkdir($folder, 0775, true);
        }

        file_put_contents($folder . "{$DSCOMANDA} - {$CDSENHAPED} .txt", $impSimulada, FILE_APPEND);

        if(!$multiplasComandas){
            $comandos->cutPaper();
        }
        return array('error' => false);
    }

    private function removeAcentos($texto) {
        $table = array(
            'á' => 'a', 'à' => 'a', 'ã' => 'a', 'â' => 'a',
            'é' => 'e', 'ê' => 'e', 'í' => 'i', 'ó' => 'o', 'ô' => 'o',
            'õ' => 'o', 'ú' => 'u', 'ü' => 'u', 'ç' => 'c', 'Á' => 'A',
            'À' => 'A', 'Ã' => 'A', 'Â' => 'A', 'É' => 'E', 'Ê' => 'E',
            'Í' => 'I', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O', 'Ú' => 'U',
            'Ü' => 'U', 'Ç' => 'C_'
        );
        return preg_replace("[^a-zA-Z0-9_]", "", strtr($texto, $table));
    }

    private function printHeader($text, $printerParams, $tipoVenda, $DSCOMANDA, $nrMesa,
                                 $nmSala, $CDVENDEDOR, $NMCONSUMIDOR, $NMFANVEN, $tipoImpressao, $toGoString, $comandos, $compactPrint){
        // Monta o cabeçalho do pedido.
        //********************************************************************************
        if (!$compactPrint) 
        $text .= $this->imprimeLinha($printerParams);
        $title = $tipoImpressao == 'P' ? 'PEDIDO' : 'RELATORIO DO PUXA';
        $text .= $this->centraliza($printerParams, $title) . $printerParams['comandoEnter'];
        $data = new \DateTime('NOW');
        $text .= $this->centraliza($printerParams, 'DATA: ' . $data->format('d/m/Y H:i:s')) . $printerParams['comandoEnter'];
        $text .= $this->imprimeLinha($printerParams);

        if ($tipoVenda == 'C') {
            $text .= $this->centraliza($printerParams, 'VENDA COMANDA', $printerParams['largura']) . $printerParams['comandoEnter'];
            if (!empty($NMFANVEN)){
                $text .= 'OPERADOR: ' . $NMFANVEN . $printerParams['comandoEnter'];
            }
            if (!empty($NMCONSUMIDOR)){
                $text .= $this->centraliza($printerParams, 'CONSUMIDOR: ' . $NMCONSUMIDOR, $printerParams['largura']) . $printerParams['comandoEnter'];
            }
            $text .= 'COMANDA.: ' . $DSCOMANDA . $printerParams['comandoEnter'];
            $text .= 'MESA....: ' . $nrMesa . $printerParams['comandoEnter'];
        }
        else {
            if (!empty($nrMesa)) {
                $text .= $this->centraliza($printerParams, 'VENDA MESA', $printerParams['largura']) . $printerParams['comandoEnter'];
            }
            
            $text .= 'GARCOM..: ' . self::removeAcentos($NMFANVEN) . $printerParams['comandoEnter'];
            if ($tipoVenda == 'M') {
                $text .= 'AMBIENTE: ' . $nmSala . $printerParams['comandoEnter'];
                $text .= 'MESA....: ' . $nrMesa . $printerParams['comandoEnter'];
            }
            if ($tipoVenda == 'B') {
                $text .= 'ORIGEM: VENDA BALCAO' . $printerParams['comandoEnter'];
            }
            
        }

        if ($toGoString != ''){
            $text .= $toGoString . $printerParams['comandoEnter'];
        }
        if (!$compactPrint) $text .= $this->imprimeLinha($printerParams);
        //********************************************************************************
        return $text;
    }

    private function printFooter($text, $printerParams, $CDSENHAPED, $compactPrint){
        // Monta rodapé.
        //********************************************************************************
        $text .= $this->imprimeLinha($printerParams);
        $text .= $this->centraliza($printerParams, 'SENHA ' . $CDSENHAPED) . $printerParams['comandoEnter'];
        $text .= $this->imprimeLinha($printerParams);
        //********************************************************************************
        return $text;
    }

    private function buscaNomePorPosicao($produtos, $posicoes){
        if(isset($produtos['CDFILIAL'])) {
            $nmPosicao = $this->printFactory->getNameByPosition($produtos, $posicoes);
            return array_column($nmPosicao, null, 'NRLUGARMESA');
        } else {
            return array();
        }
    }

    private function verificaImpUmaVez($arrayProdutos){
        foreach ($arrayProdutos as $key => &$product) {
            if ($product['IDIMPRODUVEZ'] === 'S' && $product['QTPRODCOMVEN'] > 1 && $product['IDPESAPROD'] === 'N') {
                $quantidade = $product['QTPRODCOMVEN'];
                $arrayProdutos[$key]['QTPRODCOMVEN'] = 1;
                $product['QTPRODCOMVEN'] = 1;
                for ($i = 0; $i < $quantidade - 1; $i++){
                    array_splice($arrayProdutos, $key, 0, array($product));
                }
            }
        }
        return $arrayProdutos;
    }

    private function alteraOrdemImpressao($produtos){
        $separados = array();
        $juntos = array();
        foreach ($produtos as $produto){
            if ($produto['IDIMPRODUVEZ'] === 'S'){
                array_push($separados, $produto);
            }
            else {
                array_push($juntos, $produto);
            }
        }

        return array_merge($juntos, $separados);
    }

    private function indexaProdutosPorImpressora($CDFILIAL, $CDLOJA, $produtos) {
        $impressoraProducao = array();
        $impressoraProducao2 = array();
        $impressoraPuxa = array();
        foreach ($produtos as $produtoAtual) {
            if (empty($produtoAtual['ORDEM'])) {
                $produtoAtual['ORDEM'] = 0;
            }
            if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
                $produtoAtual['IDIMPPROMOCAO'] = 'S';
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                );
                $impressoras = $this->printFactory->getPrinterByProd($CDFILIAL, $CDLOJA, $produtoAtual['CDPRODUTO']);
                $this->buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            } else {
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'CDPRODUTO' => $produtoAtual['CDPRODUTO']
                );
                $impressoras = $this->printFactory->getPrinterByProd($CDFILIAL, $CDLOJA, $produtoAtual['CDPRODUTO']);
                $this->buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, $impressoraProducao, $impressoraProducao2, $impressoraPuxa);
            }
        }
        foreach ($impressoraProducao as &$imp) {
            if(!isset($imp['NRSEQIMPRLOJABK'])){
                $imp['NRSEQIMPRLOJABK'] = '';
                $imp['IDMODEIMPRESBK'] = '';
                $imp['DSIPIMPRBK'] = '';
                $imp['CDPORTAIMPRBK'] = '';
            } else {
                if(!empty($imp['NRSEQIMPRLOJABK'])){
                    $imp['IDMODEIMPRESBK'] = $imp['IDMODEIMPRESBK'];
                    $imp['DSIPIMPRBK'] = $imp['DSIPIMPRBK'];
                    $imp['CDPORTAIMPRBK'] = $imp['CDPORTAIMPRBK'];
                }
            }
        }
        foreach ($impressoraProducao2 as &$imp) {
            if(!isset($imp['NRSEQIMPRLOJABK'])){
                $imp['NRSEQIMPRLOJABK'] = '';
                $imp['IDMODEIMPRESBK'] = '';
                $imp['DSIPIMPRBK'] = '';
                $imp['CDPORTAIMPRBK'] = '';
            } else {
                if(!empty($imp['NRSEQIMPRLOJABK'])){
                    $imp['IDMODEIMPRESBK'] = $imp['IDMODEIMPRESBK'];
                    $imp['DSIPIMPRBK'] = $imp['DSIPIMPRBK'];
                    $imp['CDPORTAIMPRBK'] = $imp['CDPORTAIMPRBK'];
                }
            }
        }
        foreach ($impressoraPuxa as &$imp) {
            if(!isset($imp['NRSEQIMPRLOJABK'])){
                $imp['NRSEQIMPRLOJABK'] = '';
                $imp['IDMODEIMPRESBK'] = '';
                $imp['DSIPIMPRBK'] = '';
                $imp['CDPORTAIMPRBK'] = '';
            } else {
                if(!empty($imp['NRSEQIMPRLOJABK'])){
                    $imp['IDMODEIMPRESBK'] = $imp['IDMODEIMPRESBK'];
                    $imp['DSIPIMPRBK'] = $imp['DSIPIMPRBK'];
                    $imp['CDPORTAIMPRBK'] = $imp['CDPORTAIMPRBK'];
                }
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function buscaImpressoraPorProduto($impressoras, $produtoAtual, $CDFILIAL, $CDLOJA, &$impressoraProducao, &$impressoraProducao2, &$impressoraPuxa){
        if (!empty($produtoAtual['NRSEQIMPRLOJA'])) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRSEQIMPRLOJA' => $produtoAtual['NRSEQIMPRLOJA']
            );
            $dadosImpressora = $this->printFactory->getPrinterDetails($CDFILIAL, $CDLOJA, $produtoAtual['NRSEQIMPRLOJA']);
            if (!empty($impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']])) {
                array_push($impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']]['produtos'], $produtoAtual);
            } else {
                $impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']] = array(
                    'NRSEQIMPRLOJA' => $dadosImpressora['NRSEQIMPRLOJA'],
                    'IDMODEIMPRES' => $dadosImpressora['IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'CDPORTAIMPR' => $dadosImpressora['CDPORTAIMPR'],
                    'DSENDPORTA' => $dadosImpressora['DSENDPORTA'],
                    'DSIPIMPR' => $dadosImpressora['DSIPIMPR'],
                    'DSIPPONTE' => $dadosImpressora['DSIPPONTE'],
                    'NMIMPRLOJA' => $dadosImpressora['NMIMPRLOJA'],
                    'produtos' => array($produtoAtual),
                    'NRSEQIMPRLOJABK' => null,
                    'IDMODEIMPRESBK' => null,
                    'DSIPIMPRBK' => null,
                    'CDPORTAIMPRBK' => null
                );

                if(!empty($impressoras['A_NRSEQIMPPRDBKP'])){
                    $impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']]['NRSEQIMPRLOJABK'] = $impressoras['A_NRSEQIMPPRDBKP'];
                    $impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']]['IDMODEIMPRESBK'] = $impressoras['A_IDMODEIMPRESBK'];
                    $impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']]['DSIPIMPRBK'] = $impressoras['A_DSIPIMPRBK'];
                    $impressoraProducao[$dadosImpressora['NRSEQIMPRLOJA']]['CDPORTAIMPRBK'] = $impressoras['A_CDPORTAIMPRBK'];
                }
            }
        } else {
            $impressoraPorAmbiente = $this->buscaImpressoraPorAmbiente($CDFILIAL, $CDLOJA, $produtoAtual);
            if (!empty($impressoraPorAmbiente['NRSEQIMPRLOJA'])) {
                if (!empty($impressoraProducao[$impressoraPorAmbiente['NRSEQIMPRLOJA']])) {
                    array_push($impressoraProducao[$impressoraPorAmbiente['NRSEQIMPRLOJA']]['produtos'], $produtoAtual);
                } else {
                    $impressoraProducao[$impressoraPorAmbiente['NRSEQIMPRLOJA']] = array(
                        'NRSEQIMPRLOJA' => $impressoraPorAmbiente['NRSEQIMPRLOJA'],
                        'IDMODEIMPRES' => $impressoraPorAmbiente['IDMODEIMPRES'],
                        'tipoImpressao' => 'P',
                        'CDPORTAIMPR' => $impressoraPorAmbiente['CDPORTAIMPR'],
                        'DSENDPORTA' => $impressoraPorAmbiente['DSENDPORTA'],
                        'DSIPIMPR' => $impressoraPorAmbiente['DSIPIMPR'],
                        'DSIPPONTE' => $impressoraPorAmbiente['DSIPPONTE'],
                        'NMIMPRLOJA' => $impressoraPorAmbiente['NMIMPRLOJA'],
                        'produtos' => array($produtoAtual),
                        'NRSEQIMPRLOJABK' => null,
                        'IDMODEIMPRESBK' => null,
                        'DSIPIMPRBK' => null,
                        'CDPORTAIMPRBK' => null
                    );
                }
            } else if (!empty($impressoras['A_NRSEQIMPRPROD'])) {
                if (!empty($impressoraProducao[$impressoras['A_NRSEQIMPRPROD']])) {
                    array_push($impressoraProducao[$impressoras['A_NRSEQIMPRPROD']]['produtos'], $produtoAtual);
                } else {
                    $impressoraProducao[$impressoras['A_NRSEQIMPRPROD']] = array(
                        'NRSEQIMPRLOJA' => $impressoras['A_NRSEQIMPRPROD'],
                        'IDMODEIMPRES' => $impressoras['A_IDMODEIMPRES'],
                        'tipoImpressao' => 'P',
                        'CDPORTAIMPR' => $impressoras['A_CDPORTAIMPR'],
                        'DSENDPORTA' => $impressoras['A_DSENDPORTA'],
                        'DSIPIMPR' => $impressoras['A_DSIPIMPR'],
                        'DSIPPONTE' => $impressoras['A_DSIPPONTE'],
                        'NMIMPRLOJA' => $impressoras['A_NMIMPRLOJA'],
                        'produtos' => array($produtoAtual),
                        'NRSEQIMPRLOJABK' => null,
                        'IDMODEIMPRESBK' => null,
                        'DSIPIMPRBK' => null,
                        'CDPORTAIMPRBK' => null,
                    );
                }

                if(!empty($impressoras['A_NRSEQIMPPRDBKP'])){
                    $impressoraProducao[$impressoras['A_NRSEQIMPRPROD']]['NRSEQIMPRLOJABK'] = $impressoras['A_NRSEQIMPPRDBKP'];
                    $impressoraProducao[$impressoras['A_NRSEQIMPRPROD']]['IDMODEIMPRESBK'] = $impressoras['A_IDMODEIMPRESBK'];
                    $impressoraProducao[$impressoras['A_NRSEQIMPRPROD']]['DSIPIMPRBK'] = $impressoras['A_DSIPIMPRBK'];
                    $impressoraProducao[$impressoras['A_NRSEQIMPRPROD']]['CDPORTAIMPRBK'] = $impressoras['A_CDPORTAIMPRBK'];
                }
            }
        }

        if (!empty($impressoras['B_NRSEQIMPRPROD2'])) {
            if (!empty($impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']])) {
                array_push($impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']]['produtos'], $produtoAtual);
            } else {
                $impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']] = array(
                    'NRSEQIMPRLOJA' => $impressoras['B_NRSEQIMPRPROD2'],
                    'IDMODEIMPRES' => $impressoras['B_IDMODEIMPRES'],
                    'tipoImpressao' => 'P',
                    'CDPORTAIMPR' => $impressoras['B_CDPORTAIMPR'],
                    'DSENDPORTA' => $impressoras['B_DSENDPORTA'],
                    'DSIPIMPR' => $impressoras['B_DSIPIMPR'],
                    'DSIPPONTE' => $impressoras['B_DSIPPONTE'],
                    'NMIMPRLOJA' => $impressoras['B_NMIMPRLOJA'],
                    'produtos' => array($produtoAtual),
                    'NRSEQIMPRLOJABK' => null,
                    'IDMODEIMPRESBK' => null,
                    'DSIPIMPRBK' => null,
                    'CDPORTAIMPRBK' => null
                );

                if(!empty($impressoras['B_NRSEQIMPPRD2BKP'])){
                    $impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']]['NRSEQIMPRLOJABK'] = $impressoras['B_NRSEQIMPPRD2BKP'];
                    $impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']]['IDMODEIMPRESBK'] = $impressoras['B_IDMODEIMPRESBK'];
                    $impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']]['DSIPIMPRBK'] = $impressoras['B_DSIPIMPRBK'];
                    $impressoraProducao2[$impressoras['B_NRSEQIMPRPROD2']]['CDPORTAIMPRBK'] = $impressoras['B_CDPORTAIMPRBK'];
                }
            }
        }

        if (!empty($impressoras['C_NRSEQIMPRPUXA'])) {
            if (!empty($impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']])) {
                array_push($impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']]['produtos'], $produtoAtual);
            } else {
                $impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']] = array(
                    'NRSEQIMPRLOJA' => $impressoras['C_NRSEQIMPRPUXA'],
                    'IDMODEIMPRES' => $impressoras['C_IDMODEIMPRES'],
                    'tipoImpressao' => 'U',
                    'CDPORTAIMPR' => $impressoras['C_CDPORTAIMPR'],
                    'DSENDPORTA' => $impressoras['C_DSENDPORTA'],
                    'DSIPIMPR' => $impressoras['C_DSIPIMPR'],
                    'DSIPPONTE' => $impressoras['C_DSIPPONTE'],
                    'NMIMPRLOJA' => $impressoras['C_NMIMPRLOJA'],
                    'produtos' => array($produtoAtual),
                    'NRSEQIMPRLOJABK' => null,
                    'IDMODEIMPRESBK' => null,
                    'DSIPIMPRBK' => null,
                    'CDPORTAIMPRBK' => null
                );

                if(!empty($impressoras['C_NRSEQIMPPUXABKP'])){
                    $impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']]['NRSEQIMPRLOJABK'] = $impressoras['C_NRSEQIMPPUXABKP'];
                    $impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']]['IDMODEIMPRESBK'] = $impressoras['C_IDMODEIMPRESBK'];
                    $impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']]['DSIPIMPRBK'] = $impressoras['C_DSIPIMPRBK'];
                    $impressoraPuxa[$impressoras['C_NRSEQIMPRPUXA']]['CDPORTAIMPRBK'] = $impressoras['C_CDPORTAIMPRBK'];
                }
            }
        }
        return array(
            'impressoraProducao' => $impressoraProducao,
            'impressoraProducao2' => $impressoraProducao2,
            'impressoraPuxa' => $impressoraPuxa
        );
    }

    private function buscaImpressoraPorAmbiente($CDFILIAL, $CDLOJA, $produtoAtual) {
        $impressoraPorAmbiente = array();
        if (!empty($produtoAtual['CDSALA']) && empty($produtoAtual['NRSEQIMPRLOJA'])) {
            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'CDPRODUTO' => $produtoAtual['CDPRODUTO'],
                'NRCONFTELA' => $produtoAtual['NRCONFTELA'],
                'DTINIVIGENCIA' => new \DateTime($produtoAtual['DTINIVIGENCIA']),
                'CDAMBIENTE' => $produtoAtual['CDSALA']
            );
            $types = array(
                'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
            );
            $retornoQuery = $this->printFactory->getPrinterByEnvironment($CDFILIAL, $CDLOJA, $produtoAtual);
            if (sizeof($retornoQuery) === 1) {
                $impressoraPorAmbiente = $retornoQuery;
            }
        }
        return $impressoraPorAmbiente;
    }

    private function getDadosImpressora($CDFILIAL, $CDCAIXA){

        $dadosImpressora = $this->printFactory->getPrinterData($CDFILIAL, $CDCAIXA);
        $dadosImpressora['IDUTILIMPFRONT'] = 'N';
        $result = array(
            'error' => false,
            'dadosImpressora' => $dadosImpressora
        );
        return $result;
    }

    public function buscaParametrosImpressora($IDMODEIMPRES) {
        /*
        1 - Bematech Mp-20
        2 - Epson Tm-T88IIP
        3 - Bematech Mp-2100
        4 - Bematech Mp-4000
        5 - Sewoo - Chile
        6 - IBM-4610 - Colombia
        7 - Bematech MP-100 - Peru
        8 - Daruma DR600
        9 - Epson TM-U22A
        10 - Daruma DR700
        11 - Epson M244A
        12 - Zebra Tlp 2844
        13 - Bematech MP-4200
        14 - Epson TM-T88V
        16 - Epson TM-T20
        17 - Sweda SI-300S
        18 - Elgin I9
        19 - Epson TM-T88IV Ethernet
        20 - Elgin Kiosk BKT681
        21 - Bematech MP-4200 - Compartilhada
        26 - Poynt
        23 - Custom TG2480H
        25 - GPOS700
        30 - Tanca TP-650
        38 - Getnet
        39 - Pagseguro
        40 - Stone
        44 - Sweda SI-300-SIX
        */

        switch ($IDMODEIMPRES) {
            case '1':
                // Bematech Mp-20
                $params = array(
                    'largura' => 36,
                    'larguraCupom' => 36,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '2':
                // Epson Tm-T88IIP
                $params = array(
                    'largura' => 42 ,
                    'larguraCupom' => 42 ,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '3':
                // Bematech Mp-2100
                $params = array(
                    'largura' => 48+17,
                    'larguraCupom' => 48+17,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '4':
                // Bematech Mp-4000
                $params = array(
                    'largura' => 64,
                    'larguraCupom' => 64,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '5':
                // Sewoo - Chile
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 42,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '6':
                // IBM-4610 - Colombia
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 42,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '7':
                // Bematech MP-100 - Peru
                $params = array(
                    'largura' => 40,
                    'larguraCupom' => 40,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '8':
                // Daruma DR600
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '9':
                // Epson TM-U22A
                $params = array(
                    'largura' => 40,
                    'larguraCupom' => 40,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '10':
                // Daruma DR700
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '11':
                // Epson M244A
                $params = array(
                    'largura' => 40,
                    'larguraCupom' => 40,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '12':
                // Zebra Tlp 2844
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '13':
                // Bematech MP-4200
                $params = array(
                    'largura' => 50,
                    'larguraCupom' => 64,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '14':
                // Epson TM-T88V
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 56,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '16':
                // Epson TM-T20
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 64,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '17':
                // Sweda SI-300S
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 42,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '18':
                // Elgin I9
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '19':
                // Epson TM-T88IV Ethernet
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 56,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '20':
                // 20 - Elgin Kiosk BKT681
                $params = array(
                    'largura' => 59,
                    'larguraCupom' => 59,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => false
                );
                break;
            case '21':
                // Bematech MP-4200 - Compartilhada
                $params = array(
                    'largura' => 42,
                    'larguraCupom' => 64,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '23':
                $params = array(
                    'largura' => 43,
                    'larguraCupom' => 43,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => chr(27) . chr(105),
                    'impressaoFront' => false
                );
                break;
            case '25':
                // 25 - GPOS700 - Gertec
                $params = array(
                    'largura' => 38,
                    'larguraCupom' => 38,
                    'tipoLetra' => 1,
                    'comandoEnter' => "\n",
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '26':
                // 26 - Poynt
                $params = array(
                    'largura' => 32,
                    'larguraCupom' => 32,
                    'tipoLetra' => 1,
                    'comandoEnter' => '\n',
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '27':
                // 27 - CieloLio v2
                $params = array(
                    'largura' => 37,
                    'larguraCupom' => 37,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '30':
                // 30 - Tanca TP-650
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 3,
                    'comandoEnter' => chr(13) . chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '38':
                // 38 - Getnet
                $params = array(
                    'largura' => 32,
                    'larguraCupom' => 32,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '39':
                // 39 - PagSeguro
                $params = array(
                    'largura' => 37,
                    'larguraCupom' => 37,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '40':
                // 40 - Stone
                $params = array(
                    'largura' => 37,
                    'larguraCupom' => 37,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '44':
                // Sweda SI-300-SIX
                $params = array(
                    'largura' => 56,
                    'larguraCupom' => 56,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(109),
                    'impressaoFront' => false
                );
                break;
            case '46':
                //
                $params = array(
                    'largura' => 48,
                    'larguraCupom' => 48,
                    'tipoLetra' => 2,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => false
                );
                break;
            case '47':
                // 47 - MiniPDV Acoplada - Elgin/Bematech
                $params = array(
                    'largura' => 32,
                    'larguraCupom' => 32,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => null,
                    'impressaoFront' => true
                );
                break;
            case '48':
                //
                $params = array(
                    'largura' => 32,
                    'larguraCupom' => 32,
                    'tipoLetra' => 1,
                    'comandoEnter' => chr(10),
                    'comandoCortePapel' => chr(27) . chr(119),
                    'impressaoFront' => true
                );
                break;

            default:
            $params = array(
                'largura' => 49,
                'larguraCupom' => 49,
                'tipoLetra' => 3,
                'comandoEnter' => chr(13) . chr(10),
                'comandoCortePapel' => chr(27) . chr(119),
                'impressaoFront' => false
                );
                break;
        }
        return $params;
    }

    public function impressaoPedidos($dadosImpressora, Command $comandos, $nrPedido = '0') {
        try {
            $result = array('error'=> false);
            if ($this->utilizaImpressaoPonte) {
                $impressao = $this->configuracaoPonte($dadosImpressora, $comandos);
                $result = $impressao->sendPrint();
            }else{
                $text = '';
                $retorno = array();

                foreach ($comandos->getCommands() as $key => $comando) {
                    switch ($comando['name']) {
                        case 'text':
                            $text .= $comando['parameters']['text'];
                            $retorno['error'] = false;
                        break;
                        case 'cutPaper':
                            $retorno = $this->imprimeNaoFiscal($text, $dadosImpressora, $nrPedido.'P');
                            $text = '';
                        break;
                        default: $retorno['error'] = false;
                    }
                    if ($retorno['error']) {
                        $result = $retorno;
                    }
                }
            }
        } catch(Exception $e) {
            //$this->util->logImpressao($e->getMessage());
            $result = array(
                'error' => true,
                'message' => $dadosImpressora['NMIMPRLOJA'] . ': ' . $e->getMessage()
            );
        }
        return $result;
    }

    private function configuracaoPonte($dadosImpressora, Command $comandos) {
        $porta = !empty($dadosImpressora['DSIPIMPR']) ? $dadosImpressora['DSIPIMPR'] : $dadosImpressora['CDPORTAIMPR'];
        $impressora = new Printing(null, $comandos);
        $propriedadesImpressora = array(
            'printerType' => $dadosImpressora['IDMODEIMPRES'],
            'kioskPort' => 0,
            'port' => $porta,
            'impBackup' => false
        );

        if((isset($dadosImpressora['IDMODEIMPRESBK'])) && (isset($dadosImpressora['CDPORTAIMPRBK']) || isset($dadosImpressora['DSIPIMPRBK'])) ){
            if(!empty($dadosImpressora['IDMODEIMPRESBK'])){
                $portabk = !empty($dadosImpressora['DSIPIMPRBK']) ? $dadosImpressora['DSIPIMPRBK'] : $dadosImpressora['CDPORTAIMPRBK'];
                $propriedadesImpressora['impBackup'] = array(
                    'printerType' => $dadosImpressora['IDMODEIMPRESBK'],
                    'port' => $portabk
                );
            }
        }

        $impressao =  $impressora->setPrinterProperties($propriedadesImpressora);
        $urlPonte = !empty($dadosImpressora['DSIPPONTE']) ?  $dadosImpressora['DSIPPONTE'] : null;
        $impressao->setUrl($urlPonte);
        return $impressao;
    }

    public function imprimeNaoFiscal($text, $dadosImpressora, $sequencialArquivo = false) {
        global $CLIENT_NAME;
        $this->logPath = $CLIENT_NAME['logPath'];
        try {
            $printerProperties = $this->buscaParametrosImpressora($dadosImpressora['IDMODEIMPRES']);
            $text = $this->formataMp20($dadosImpressora['IDMODEIMPRES'], $text);

            if ($this->utilizaImpressaoPonte) {
                $comandos = new Command();
                $printerProperties['letterType'] = $printerProperties['tipoLetra'];
                $comandos->text($text, $printerProperties);
                $comandos->cutPaper();
                $issaas = false;
                $dadosImpressora['IDUTILIMPFRONT'] = isset($dadosImpressora['IDUTILIMPFRONT'])?$dadosImpressora['IDUTILIMPFRONT']:false;
                if($issaas || ($dadosImpressora['IDUTILIMPFRONT']===true||$dadosImpressora['IDUTILIMPFRONT']==='S')){
                    $result = array(
                        'saas'      => true,
                        'IDUTILIMPFRONT' => ($dadosImpressora['IDUTILIMPFRONT']===true||$dadosImpressora['IDUTILIMPFRONT']==='S'),
                        'impressora'=> $dadosImpressora,
                        'comandos'  => $comandos->getCommands(),
                        'error'     => false);
                }else{
                    //$result = $this->requisicaoPonte($dadosImpressora, $comandos);
                    $result['saas'] = false;
                }
            } else {
                $folder = $this->logPath;
                if ($this->util->createFolder($folder)) {
                    if ($this->deveFazerMapeamento($dadosImpressora['DSENDPORTA'], $dadosImpressora['DSIPIMPR'])) {
                        $this->fazMapeamento($dadosImpressora['DSENDPORTA'], $dadosImpressora['CDPORTAIMPR']);
                    }
                    if ($this->utilizaImpressaoPHP) {
                        // impressao via PHP usando portas do Windows
                        $result = $this->impressaoPhp($dadosImpressora, $printerProperties, $text);
                    }
                } else {
                    $result = array(
                        'error' => true,
                        'message' => 'Não foi possível criar o diretório de impressão, verifique as permissões.'
                    );
                }
            }
        } catch(Exception $e) {
            //$this->util->logImpressao($e->getMessage());
            $result = array(
                'error' => true,
                'message' => 'Não foi possível imprimir. Verifique a conexão com a impressora.',
                'exceptionMessage' => $e->getMessage()
            );
        }
        return $result;
    }

    private function impressaoPhp($dadosImpressora, $printerProperties, $text) {
        $porta = !empty($dadosImpressora['DSIPIMPR']) ? $dadosImpressora['DSIPIMPR'] : $dadosImpressora['CDPORTAIMPR'];
        $text .= $printerProperties['comandoCortePapel'];
        $printerStream = fopen($porta, 'w');
        fwrite($printerStream, $text);
        fclose($printerStream);
        return array('error' => false);
    }

    private function deveFazerMapeamento($DSENDPORTA, $DSIPIMPR) {
        return !empty($DSENDPORTA) && empty($DSIPIMPR);
    }

    private function fazMapeamento($DSENDPORTA, $CDPORTAIMPR) {
        if (!empty($DSENDPORTA)) {
            $comandoMapeamento = 'NET USE ' . $CDPORTAIMPR . ' \\\\' . $DSENDPORTA . ' /PERSISTENT:YES';
            $handle = new \COM('WScript.Shell');
            $handle->Run($comandoMapeamento, 0, false);
        }
    }

    public function formataMp20($IDMODEIMPRES, $string) {
        if ($IDMODEIMPRES == '1') {
            $printerParams = self::buscaParametrosImpressora($IDMODEIMPRES);
            //Quantidade de espaços a serem dados
            for ($i = 0; $i < 4; $i++) {
                $string .= $printerParams['comandoEnter'];
            }
        }
        return $string;
    }

    public function imprimeLinhaCommand($printerParams, $commands, $text = '-'){
        $line = $this->imprimeLinha($printerParams, $text);
        $commands->text($line, $printerParams);
    }
    public function imprimeLinha($printerParams, $text = '-') {
        //Função criada para não deixar imprimir duas linhas em sequencia
        $linha = self::replicate($text, $printerParams['largura']) . $printerParams['comandoEnter'];
        return $linha;
    }

    public function checaEnter(&$texto, $printerParams) {
        //Função criada para não deixar dois enters seguidos
        if(substr($texto, -(strlen($printerParams['comandoEnter']))) != $printerParams['comandoEnter']){
            $texto .= $printerParams['comandoEnter'];
        }
    }

    public function quebraLinha($texto, $printerParams, $cortaPalavra){
        return wordwrap($texto, $printerParams['largura'], $printerParams['comandoEnter'], $cortaPalavra);
    }

    public function preencheLinha($printerParams, $text, $caracter, $largura = null) {
        if (empty($largura)) {
            $largura = $printerParams['largura'];
        }
        $nrEspaços =  $largura-strlen($text);
        if ($nrEspaços > 0){
            $linha = $text.self::replicate($caracter, $nrEspaços) . $printerParams['comandoEnter'];
        }else{
            $linha = $text;
        }
        return $linha;
    }

    public function preenche($largura, $text, $caracter) {
        $nrEspaços =  $largura - strlen($text);
        if ($nrEspaços > 0){
            $linha = $text.self::replicate($caracter, $nrEspaços);
        }else{
            $linha = $text;
        }
        return $linha;
    }

    public function alinhaInicioFim($printerParams, $textoInicio, $textoFim) {
        $largura = $printerParams['largura'];

        $numEspacos = $largura - (strlen($textoInicio) + strlen($textoFim));
        $espaco = self::replicate(' ', $numEspacos);

        return $this->quebraLinha($textoInicio . $espaco . $textoFim, $printerParams, false);
    }

    public function replicate($char, $numero) {
        $result = '';
        for ($i = 0; $i < $numero; ++$i){
            $result .= $char;
        }
        return $result;
    }

    public function centraliza($printerParams, $string, $largura = null) {
        $result = '';
        $tamanhoString = strlen($string);

        if ($largura == null) {
            $largura = $printerParams['largura'];
        }

        if ($tamanhoString <= $largura){
            $espacos = floor($largura / 2);
            $len = floor($tamanhoString / 2);
            $result = self::replicate(' ', $espacos - $len) . $string;
        } else if ($tamanhoString > $largura){
            $arrString = explode(' ', $string);
            $linha = '';

            foreach ($arrString as $key => $splitedString) {
                $len = strlen($splitedString);
                if ($len === $largura) {
                    $result .= $splitedString;
                    if (isset($arrString[$key + 1])) {
                        $result .= $printerParams['comandoEnter'];
                    }
                } else if ($len < $largura) {
                    $linha .= $splitedString;
                    if (isset($arrString[$key + 1])) {
                        if ((strlen($linha) + 1 + strlen($arrString[$key + 1])) >= $largura){
                            $result .= self::replicate(' ', floor(($largura - strlen($linha))/2)) . $linha . $printerParams['comandoEnter'];
                            $linha = '';
                        } else {
                            $linha .= ' ';
                        }
                    } else {
                        $result .= self::replicate(' ', floor(($largura - strlen($linha))/2)) . $linha;
                    }
                } else {
                    // busca string da última linha
                    $centraliza = substr($splitedString, -($len % $largura));
                    // 'comandoEnter' nas linhas que não serão centralizadas
                    $result .= self::quebraLinha(explode($centraliza, $splitedString)[0], $printerParams, true) . $printerParams['comandoEnter'];

                    if (isset($arrString[$key + 1])) {
                        if ((strlen($centraliza) + 1 + strlen($arrString[$key + 1])) >= $largura){
                            $result .= self::replicate(' ', floor(($largura - strlen($centraliza))/2)) . $centraliza . $printerParams['comandoEnter'];
                        } else {
                            $linha = $centraliza . ' ';
                        }
                    } else {
                        $result .= self::replicate(' ', floor(($largura - strlen($centraliza))/2)) . $centraliza;
                    }
                }
            }
        }

        return $result;
    }
}